class Decider:
    def __init__(self, device_count, final_preference, exclusions):
        """
        The object that stores the values for the number of devices remaining and the user device preferences,
        so that the decide_preferences method can access them.
        :param device_count: The dict full of device names as keys with numbers of devices as values
        :param final_preference: The list of individual lists belonging to each participant
        """
        self.device_count = device_count
        self.final_preference = final_preference
        self.exclusions = exclusions
        self.winners = {}
        self.drawn_again = []
        self.no_prises_matched = []
        self.num_first_pick = 0
        self.num_second_pick = 0
        self.num_third_pick = 0
        self.num_fourth_pick = 0
        self.num_last_pick = 0

    def determine_pick(self, i2_level):
        if i2_level == 0:
            self.num_first_pick += 1
        if i2_level == 1:
            self.num_second_pick += 1
        if i2_level == 2:
            self.num_third_pick += 1
        if i2_level == 3:
            self.num_last_pick += 1

    def decide_preference(self, i, p, i2_level):
        """
        Will assign participants a device based on their preferences, but only if there are enough devices left.
        If all devices of their 0th preference are gone, it will assign a device from their 1st preference, then 2nd
        repeating until it finds a device count type that is not 0. If the user has already won, it skips that user and
        records the users name as drawn_again.
        :param i: the int index in the preference list belonging to the participant
        :param p: the string name of the participant
        :param i2_level: the int index of the preference sub-list as well as the level of recursion
        :return: none
        """
        if self.winners.get(p):
            print("User {0} already won, skipping".format(p))
            self.drawn_again.append(p)
        else:
            if self.final_preference[i][i2_level] == "Laptop":
                if self.device_count["num_laptop"] > 0:
                    self.winners[p] = self.final_preference[i][i2_level]
                    self.device_count["num_laptop"] -= 1
                    self.determine_pick(i2_level)
                else:
                    if self.final_preference[i2_level +1 ]:
                        self.decide_preference(i, p, i2_level + 1)
                    else:
                        self.no_prises_matched.append(p)

            if self.final_preference[i][i2_level] == "All-in-one PC":
                if self.device_count["num_aio"] > 0:
                    self.winners[p] = self.final_preference[i][i2_level]
                    self.device_count["num_aio"] -= 1
                    self.determine_pick(i2_level)
                else:
                    if self.final_preference[i2_level +1 ]:
                        self.decide_preference(i, p, i2_level + 1)
                    else:
                        self.no_prises_matched.append(p)

            if self.final_preference[i][i2_level] == "Workstation":
                if self.device_count["num_workstation"] > 0:
                    self.winners[p] = self.final_preference[i][i2_level]
                    self.device_count["num_workstation"] -= 1
                    self.determine_pick(i2_level)
                else:
                    if self.final_preference[i2_level +1 ]:
                        self.decide_preference(i, p, i2_level + 1)
                    else:
                        self.no_prises_matched.append(p)

            if self.final_preference[i][i2_level] == "Network hardware (12-48 port gigabit switch, Sonicwall firewall)":
                if self.device_count["num_network"] > 0:
                    self.winners[p] = self.final_preference[i][i2_level]
                    self.device_count["num_network"] -= 1
                    self.determine_pick(i2_level)
                else:
                    if self.final_preference[i2_level +1 ]:
                        self.decide_preference(i, p, i2_level + 1)
                    else:
                        self.no_prises_matched.append(p)

            if self.final_preference[i][i2_level] == "AV":
                if self.device_count["num_projector"] > 0:
                    self.winners[p] = self.final_preference[i][i2_level]
                    self.device_count["num_projector"] -= 1
                    self.determine_pick(i2_level)
                else:
                    if self.final_preference[i2_level +1 ]:
                        self.decide_preference(i, p, i2_level + 1)
                    else:
                        self.no_prises_matched.append(p)

